package cJTab;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import javax.swing.Icon;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

/**
 * La classe <code>CTabbedPane</code> sert  crer un <code>JTabbedPane</code> avec un bouton de fermeture et 
 * avec la meme allure que ceux de SWT.<br>
 * <span style="color: rgb(255, 0, 0); font-weight: bold;">ATTENTION</span> : Quand le gestionnaire de placement  des onglets est <code>JTabbedPane.SCROLL_TAB_LAYOUT</code>
 * et qu'il y a eut dcalage des onglets la fermeture ne marche pas !
 *
 */
public class CTabbedPane extends JTabbedPane  {

	private static final long serialVersionUID = 4651125469262127893L;

	CloseTabbedPaneUI monUI;
    
    private Color selectedColor= new Color(238, 238, 238);
    private Color noneSelectedColor =  new Color(200, 200, 200);

	/**
     * Constructeur par dfaut de <code>CTabbedPane</code>
     */
    public CTabbedPane() {
        this(new Color(238, 238, 238),  new Color(200, 200, 200));
    }
    
    /**
     * Constructeur de <code>CTabbedPane</code>
     * @param selectedColor : couleur de l'onglet selectionn
     * @param noneSelectedColor : couleur des onglets non selectionns
     */
    public CTabbedPane(Color selectedColor, Color noneSelectedColor) {
        super();
        this.selectedColor = selectedColor;
        this.noneSelectedColor = noneSelectedColor;
        
        this.setBackground(noneSelectedColor);
        this.addMouseListener(new ecouteurDeMouse());
        this.addMouseMotionListener(new ecouteurDeMouseMotion());
        
        this.monUI = new CloseTabbedPaneUI(this, this.selectedColor, this.noneSelectedColor);
        this.setUI(monUI);        
    }
    


	/**
     * Insertion d'un onglet dans le <code>CTabbedPane</code>. Si le composant implmente <code>CloseTabListener</code>, 
     * l'onglet est un onglet avec croix de fermeture, sinon c'est un onglet sans croix de fermeture.
     * 
     * @param titre : titre de l'onglet
     * @param icone : icone a afficher du cot gauche du texte
     * @param component : composant que contient l'onglet
     * @param toolTip : le conseil sur l'onglet
     * @param index : l'indice o est insr l'onglet
     */
    
    public void insertTab(String titre, Icon icone, Component component, String toolTip, int index) {
    	if(component instanceof CloseTabListener){
   		    CloseTabIcon cti =   new CloseTabIcon(icone, titre, noneSelectedColor, this, component);
            super.insertTab(titre, cti, component, toolTip, index);
   	    }
   	   else super.insertTab(titre, icone,  component, toolTip, index);  
    }
    
    /**
     * Ajout d'un onglet dans le <code>CTabbedPane</code>. Si le composant implmente <code>CloseTabListener</code>, 
     * l'onglet est un onglet avec croix de fermeture, sinon c'est un onglet sans croix de fermeture.
     * 
     * @param titre : titre de l'onglet
     * @param icone : icone a afficher du cot gauche du texte
     * @param component : composant que contient l'onglet
     * @param toolTip : le conseil sur l'onglet
     */
    
    public void addTab(String titre,  Icon icone, Component component, String toolTip){
    	if(component instanceof CloseTabListener){
    		 CloseTabIcon cti =   new CloseTabIcon(icone, titre, noneSelectedColor, this, component);
             super.addTab(titre, cti, component, toolTip);
    	}
    	else super.addTab(titre, icone,  component, toolTip);  	
    }
    
    
    /**
     * Permet de changer l'icone a gauche du texte
     * @param position l'indice de l'onglet dont on veut modifier l'icone
     * @param icone la nouvelle icone
     */
    
    public void setIconAt(int position, Icon icone) {
        if (getIconAt(position) instanceof CloseTabIcon)
            ((CloseTabIcon)getIconAt(position)).setIcon(icone);
        else
            super.setIconAt(position, icone);
    }
    
    /**
     * Classe permettant de rpondre aux clic souris se produisant sur un onglet.
     * @author bcaylux
     *
     */
    private class ecouteurDeMouse extends MouseAdapter {
    	/**
    	 * Un clic sur une croix rouge entraine la demande de fermeture de l'onglet
    	 */
        public void mousePressed(MouseEvent e) {
            int tabNumber = getUI().tabForCoordinate(CTabbedPane.this, e.getX(), e.getY());
            if (tabNumber < 0 || !(getIconAt(tabNumber) instanceof CloseTabIcon)) return;
            
            Rectangle rect=((CloseTabIcon)getIconAt(tabNumber)).getBounds(tabNumber);
 
            if(CTabbedPane.this.getTabPlacement()==JTabbedPane.BOTTOM){
            	rect.y+=CTabbedPane.this.getBoundsAt(tabNumber).y;
            }
            if (rect.contains(e.getX(), e.getY())) {
            	CloseTabListener mjp = (CloseTabListener)CTabbedPane.this.getComponentAt(tabNumber);
                if(mjp.closeAction(new CloseTabEvent(CTabbedPane.this, tabNumber)))
                    remove(tabNumber);

            }
        }
        
        
    }
    
    /**
     * Classe permettant de rpondre aux dplacement souris se produisant sur un onglet.
     * @author bcaylux
     *
     */
    
    private class ecouteurDeMouseMotion extends MouseMotionAdapter {
        public void mouseMoved(MouseEvent e) {
        	// quand la souris se dplace sur un onglet on affiche sa croix
        	// en blanc si la souris n'est pas sur la croix (setRollOved(false))
        	// en rouge si la souris est sur la croix (setRollOved(true))
            int tabNumber = getUI().tabForCoordinate(CTabbedPane.this, e.getX(), e.getY());
            if (tabNumber < 0) return;                                   // la souris n'est pas sur l'onglet 
            if( !(getIconAt(tabNumber) instanceof CloseTabIcon)) return; // l'onglet n'a pas de croix

            Rectangle rect=((CloseTabIcon)getIconAt(tabNumber)).getBounds(tabNumber);
            //Rectangle rect=((CloseTabIcon)getIconAt(tabNumber)).getBounds(); 
            if(CTabbedPane.this.getTabPlacement()==JTabbedPane.BOTTOM){
            	rect.y+=CTabbedPane.this.getBoundsAt(tabNumber).y;
            }

            if (rect.contains(e.getX(), e.getY())){
            	 ((CloseTabIcon)getIconAt(tabNumber)).setRollOved(true);
            }
            else {
                for(int i = 0 ; i < getTabCount() ; i++)
                    if(getIconAt(i) instanceof CloseTabIcon)
                        ((CloseTabIcon)getIconAt(i)).setRollOved(false);                   
            }  
            repaint();
        }
    }

    

}  



/**
 * La classe <code>CloseTabIcon</code> est urilise pour afficher l'icne de l'onglet et sa croix ...
 * @author bcaylux
 *
 */
class CloseTabIcon  implements Icon {

    private boolean isRollOved = false;  
    
    private Icon icone; // L'icone  gauche du texte ou null s'il n'y a pas d'icone
    private int li, hi; // largeur et hauteur de icone
    private Image croix1, croix2; // croix blanche et croix rouge
    private int width, height;    // largeur et hauteur de la croix
    private int xCroix, yCroix;   // coordonnes de la croix 
    private String titre;   // le titre 
    private int titreLong;  // et sa longueur (en pixels)
    private int hauteur  ;  // la hauteur de l'onglet
   
    private CTabbedPane mtp;      // le JTabbedPane
    private Component component;  // le composant correspondant a cette instance de  CloseTabIcon

    /**
     * Constructeur de <code>CloseTabIcon</code>
     * @param icone l'icone  gauche du texte
     * @param titre le texte de l'onglet
     * @param fond la couleur du fond
     * @param mtp le <code>CTabbedPane</code> auquel appartient l'onglet
     * @param component le composant correspondant  cet onglet
     */public CloseTabIcon(Icon icone, String titre, Color fond, CTabbedPane mtp, Component component) {

        this.titre = titre;
        this.icone = icone;
        if(icone==null)
             { li=0; hi=0;}
        else { li=icone.getIconWidth(); hi = icone.getIconHeight();}
        this.mtp = mtp;
        this.component = component;

        // chargement des croix 
        Toolkit tk = Toolkit.getDefaultToolkit();
        croix1 = tk.getImage(getClass().getResource("/cJTab/croix1.gif"));
        croix2 = tk.getImage(getClass().getResource("/cJTab/croix2.gif"));
        MediaTracker mt = new MediaTracker(mtp);
        mt.addImage(croix1, 0);
        mt.addImage(croix2, 0);
        try {
           mt.waitForID(0);
           width = croix1.getWidth(null);
           height = croix1.getHeight(null);
           // l'image est prsente en mmoire
        } catch(InterruptedException e) {}
            
    }
   
    
    public void setRollOved (boolean b) {
        isRollOved = b;
    }
    

    public void setIcon (Icon icone) {
        this.icone = icone;
    }
    

    public void paintIcon(Component c, Graphics g, int x, int y) {    
        // s'il y a une icone, on l'affiche
        if(icone!=null) icone.paintIcon(mtp, g, x, y);
            
        // l'onglet slectionn porte toujours sa croix
    	if(  mtp.getSelectedComponent() == component ){
            titre = mtp.getTitleAt(mtp.getSelectedIndex());
            titreLong = g.getFontMetrics().stringWidth(titre);
            
            // Calcul des coordonnes de la croix
            xCroix = x + li + titreLong + 3 ;
            yCroix = y + 1 + (icone != null? (hi-height)/2 : 0) ;
     
    		if (isRollOved){
            	// souris sur la croix, alors affichage rouge
            	g.drawImage(croix2, xCroix, yCroix, null, mtp); 
            	mtp.setToolTipTextAt(mtp.getSelectedIndex(),"Fermer "+mtp.getTitleAt(mtp.getSelectedIndex())+" !");
            }else{       	
            	if( mtp.getComponentAt(mtp.getSelectedIndex()) == component ){
                     g.drawImage(croix1, xCroix, yCroix, null, mtp);
                     mtp.setToolTipTextAt(mtp.getSelectedIndex(), mtp.getTitleAt(mtp.getSelectedIndex()));
             	}
            }
    		return ;
    	}
    	
        int tabSurvol = mtp.monUI.getRollOverTab();
        if(tabSurvol<0) return;
        if(mtp.getComponentAt(tabSurvol) != component) return;
        // re affichage de l'onglet survol ...
        titre = mtp.getTitleAt(tabSurvol);
        titreLong = g.getFontMetrics().stringWidth(titre);
        
        // Calcul des coordonnes de la croix
        xCroix = x + li + titreLong + 3 ;
        yCroix = y + 1 + (icone != null? (hi-height)/2 : 0) ;
 
        if (isRollOved){
        	// souris sur la croix, alors affichage rouge
        	g.drawImage(croix2, xCroix, yCroix, null, mtp); 
        	mtp.setToolTipTextAt(tabSurvol,"Fermer "+mtp.getTitleAt(tabSurvol)+" !");
        }else{       	
        	if( mtp.getComponentAt(tabSurvol) == component ){
                 g.drawImage(croix1, xCroix, yCroix, null, mtp);
                 mtp.setToolTipTextAt(tabSurvol, mtp.getTitleAt(tabSurvol));
         	}
        }
    }

    
    public int getIconWidth() {
        return (icone != null? icone.getIconWidth() : 0);
    }
    
    public int getIconHeight() {
        if (icone != null)
            if (icone.getIconHeight() > height)
                return icone.getIconHeight();           
        return height+1;
    }
    
    //retourne le rectangle dans lequel est inscrit la croix ...
    public Rectangle getBounds() {
    	Rectangle r = new Rectangle(xCroix, yCroix, width, height);
        return r;
    }
    
    /** Retourne le rectangle dans lequel est inscrit la croix du i<sup>me</sup> onglet...<br>
     *  (version pour le SCROLL_TAB_LAYOUT)
     * 
     * @param tabNum l'indice de l'onglet 
     * @return Le rectangle dans lequel est affiche la croix
     */
    public Rectangle getBounds(int tabNum) {
    	 Rectangle r = new Rectangle(0, 0, 0, 0);
         r = mtp.monUI.getTabBounds(tabNum, r);
         int lr = r.width;
         int la = li + titreLong + 13;
         int h = r.height;
         // le texte est centr dans l'onglet
         r.x += (lr+la)/2-width;
         // l'onglet qui suit l'onglet slectionn est un peu dcal
         if(tabNum==mtp.getSelectedIndex()+1)r.x+=7;
         r.y += 1 + (icone != null? (h-height)/2 : 0) ;
         r.width = width;
         r.height = height;
         return r;
    }
    


}



